import moment from 'moment';

import { Subject, of } from 'rxjs';
import { takeUntil, filter, take, map, catchError } from 'rxjs/operators';

import { Component, ViewChild, OnDestroy, OnInit, Inject } from '@angular/core';
import {
  Router,
  RouterEvent,
  NavigationEnd,
  PRIMARY_OUTLET,
  ActivatedRoute,
  Params
} from '@angular/router';

import { Store, select } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';
import { MatMenuTrigger } from '@angular/material/menu';
import { MatTabChangeEvent, MatTabGroup } from '@angular/material/tabs';
import { MatSidenav } from '@angular/material/sidenav';

import { User, UserInfoSS } from '@ucap/domain-organization';

import { LogService } from '@ucap/ng-logger';
import { UserSelector } from '@ucap/ng-store-organization';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import { RoomSelector } from '@ucap/ng-store-chat';

import { QueryParams as ChatQueryParams } from '@app/pages/chat/types/params.type';
import { AppAccountService } from '@app/services/app-account.service';
import { AppCallService } from '@app/services/app-call.service';
import { AppUiService } from '@app/services/app-ui.service';
import { QueryParams as OrganizationParams } from '@app/pages/organization/types/params.type';
import { AddUserDialogComponent } from '@app/sections/group/dialogs/add-user.dialog.component';
import {
  CreateDialogComponent as ChatCreateDialogComponent,
  CreateDialogData as ChatCreateDialogData,
  CreateDialogResult as ChatCreateDialogResult
} from '@app/sections/chat/dialogs/create.dialog.component';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { NativeService } from '@ucap/native';
import { AppActions } from '@app/store/actions';
import { NavigationType } from '@app/types';
import { I18nService } from '@ucap/ng-i18n';
import {
  DialpadDialogComponent,
  DialpadDialogData,
  DialpadDialogResult
} from '@app/sections/call/dialogs/dialpad.dialog.component';
import { VersionInfo } from '@ucap/domain-authentication';

const NAVS = [
  NavigationType.Group,
  NavigationType.Chat,
  NavigationType.Organization,
  NavigationType.Message,
  NavigationType.Call
];

@Component({
  selector: 'app-layouts-default',
  templateUrl: './default.layout.component.html',
  styleUrls: ['./default.layout.component.scss']
})
export class DefaultLayoutComponent implements OnInit, OnDestroy {
  @ViewChild('navTabGroup', { static: true })
  navTabGroup: MatTabGroup;

  @ViewChild('leftSidenav', { static: true })
  set leftSidenav(leftSidenav: MatSidenav) {
    this._leftSidenav = leftSidenav;
    this.appUiService.leftSidenav = leftSidenav;
  }
  get leftSidenav(): MatSidenav {
    return this._leftSidenav;
  }
  // tslint:disable-next-line: variable-name
  _leftSidenav: MatSidenav;

  @ViewChild('profileMenuTrigger', { static: true })
  profileMenuTrigger: MatMenuTrigger;

  showStatusbar = true;

  queryParams: Params;

  unreadCountChat = 0;

  /** FAB */
  fabButtonShow = true;
  fabUseCustomDefaultIcon = true; // default in this prj
  fabButtons: { icon: string; tooltip?: string; divisionType?: string }[];
  versionInfo: VersionInfo;
  user: User;

  /** Navigation */
  NavigationType = NavigationType;
  initMenu = NavigationType.Group;
  curNavi: NavigationType;

  moment = moment;

  private ngOnDestroySubject: Subject<void> = new Subject();
  constructor(
    private router: Router,
    private i18nSevice: I18nService,
    private activatedRoute: ActivatedRoute,
    private store: Store<any>,
    private appAccountService: AppAccountService,
    private appCallService: AppCallService,
    private appUiService: AppUiService,
    private logService: LogService,
    public dialog: MatDialog,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService
  ) {
    this.setFabInitial(this.initMenu);
    this.router.events
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        filter((event) => event instanceof RouterEvent)
      )
      .subscribe((event) => {
        switch (event.constructor) {
          case NavigationEnd:
            {
              const t = this.router.parseUrl((event as NavigationEnd).url);
              const p = t.root.children[PRIMARY_OUTLET];
              if (!p || !p.segments || 0 === p.segments.length) {
                break;
              }
              const index = p.segments[0].path as NavigationType;
              this.setTabGroup(index);
              this.setFabInitial(index);
            }
            break;
          default:
            break;
        }
      });

    this.activatedRoute.queryParams
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((params) => {
        this.queryParams = params;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });
  }

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(RoomSelector.unreadTotal)
      )
      .subscribe((unreadTotal) => {
        this.unreadCountChat = unreadTotal;
      });

    // this.store
    //   .pipe(
    //     takeUntil(this.ngOnDestroySubject),
    //     select(PresenceSelector.selectAllStatusBulkInfo)
    //   )
    //   .subscribe((allBulkInfo) => {
    //     // .includes(String(this.user.info.seq))
    //     if (!allBulkInfo || (!!allBulkInfo && allBulkInfo.length === 0)) {
    //       return;
    //     }
    //     const userStatusinfo = allBulkInfo.filter(
    //       (bulkInfo) => bulkInfo.userSeq === String(this.user.info.seq)
    //     )[0];

    //     const status = PresenceUtil.isOnline(userStatusinfo, PresenceType.PC);
    //     if (!status && userStatusinfo.pcStatus === StatusCode.Offline) {
    //       this.store.dispatch(
    //         PresenceActions.bulkInfo({
    //           divCd: 'myBulk',
    //           userSeqs: [String(this.user.info.seq)]
    //         })
    //       );
    //     }
    //   });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onOpenProfile(userInfo: UserInfoSS) {
    this.profileMenuTrigger.openMenu();
  }

  onSelectedTabChange(event: MatTabChangeEvent) {
    const naviMenu: NavigationType = event.tab.ariaLabel as NavigationType;
    const commands: any = [naviMenu, { outlets: { content: 'index' } }];
    const orgInitialParams: Params = {};

    // CASE :: Chat.
    if (
      naviMenu === NavigationType.Chat && // is chat.
      !!this.queryParams &&
      !!this.queryParams[ChatQueryParams.ROOM_ID]
    ) {
      // 다른 화면에서 채팅으로 바로 유입할 경우에는 navigate 초기화를 무시한다.
      this.queryParams = undefined;
      return;
    }

    // CASE :: Organization.
    if (naviMenu === NavigationType.Organization && !!this.user) {
      orgInitialParams[OrganizationParams.DEPT_SEQ] = String(
        this.user.departmentCode
      );
    }

    this.router.navigate(commands, { queryParams: orgInitialParams });
    this.appUiService.openLeftSidenavOnNarrowMode();
    this.setFabInitial(naviMenu);
  }

  onClickToggleLeftSidenav() {
    this.appUiService.toggleLeftSidenavOnNarrowMode();
  }

  setFabInitial(type: NavigationType) {
    this.curNavi = type;
    switch (type) {
      case NavigationType.Group:
        {
          this.fabButtonShow = true;
          this.fabButtons = [
            {
              icon: 'add',
              tooltip: this.i18nSevice.t('group:dialog.title.addNewBuddy'),
              divisionType: 'GROUP_NEW_ADD'
            }
          ];
        }
        break;
      case NavigationType.Chat:
        {
          this.fabButtonShow = true;
          this.fabButtons = [
            {
              icon: 'chat',
              tooltip: this.i18nSevice.t('chat:dialog.title.newChatRoom'),
              divisionType: 'CAHT_NEW_ADD'
            }
          ];

          // if (environment.productConfig.CommonSetting.useTimerRoom) {
          //   this.fabButtons.push({
          //     icon: 'timer',
          //     tooltip: this.translateService.instant('chat.newTimerChat'),
          //     divisionType: 'CHAT_NEW_TIMER_ADD'
          //   });
          // }
        }
        break;
      case NavigationType.Organization:
        {
          this.fabButtonShow = false;
        }
        break;
      case NavigationType.Message:
        {
          this.fabButtonShow = true;
          this.fabButtons = [
            {
              icon: 'add',
              tooltip: '쪽지 추가',
              divisionType: 'MESSAGE_NEW'
            }
          ];
        }
        break;
      case NavigationType.Call:
        {
          this.fabButtonShow = true;
          this.fabButtons = [
            {
              icon: 'dialpad',
              tooltip: this.i18nSevice.t('call:label.dialpad'),
              divisionType: 'OPEN_DIALPAD'
            }
          ];
        }
        break;

      default: {
        this.fabButtonShow = false;
      }
    }
  }

  onClickFab(params: { btn: any }) {
    const btn = params.btn as {
      icon: string;
      tooltip?: string;
      divisionType?: string;
    };

    switch (btn.divisionType) {
      case 'GROUP_NEW_ADD':
        {
          const dialogRef = this.dialog.open(AddUserDialogComponent, {
            panelClass: 'max-create-dialog'
          });

          dialogRef
            .afterOpened()
            .pipe(take(1))
            .subscribe(() => dialogRef.componentInstance.psUpdate());
          dialogRef
            .afterClosed()
            .pipe(
              take(1),
              map((result) => {}),
              catchError((err) => {
                return of(err);
              })
            )
            .subscribe();
        }
        break;
      case 'CAHT_NEW_ADD':
        {
          const dialogRef = this.dialog.open<
            ChatCreateDialogComponent,
            ChatCreateDialogData,
            ChatCreateDialogResult
          >(ChatCreateDialogComponent, {
            panelClass: 'max-create-dialog',
            data: {}
          });
        }
        break;
      case 'CHAT_NEW_TIMER_ADD':
        {
          // if (environment.productConfig.CommonSetting.useTimerRoom) {
          //   this.onClickNewChat('TIMER');
          // }
        }
        break;

      case 'MESSAGE_NEW':
        {
          this.logService.debug('MESSAGE_NEW');
        }
        break;

      case 'OPEN_DIALPAD':
        {
          this.dialog.open<
            DialpadDialogComponent,
            DialpadDialogData,
            DialpadDialogResult
          >(DialpadDialogComponent, {
            panelClass: 'mid-create-dialog',
            data: {}
          });
        }
        break;
    }
  }

  onClickSearch(event: Event) {
    event.stopPropagation();
  }

  onSelectedProfileManage() {
    this.profileMenuTrigger.closeMenu();
  }

  onSelectedNotice() {
    this.profileMenuTrigger.closeMenu();
  }

  onSelectedSettings() {
    this.profileMenuTrigger.closeMenu();
    this.appAccountService.dialogForSettings();
  }

  onSelectedLogout() {
    this.router.navigate(['/account/logout']);
  }

  onSelectedExit() {}

  onDoneForProfileMenu() {
    this.profileMenuTrigger.closeMenu();
  }

  onClickForOpenRoom() {
    this.nativeService.chat_openRoom('/chat/chatroom?roomId=1000');
  }

  private setTabGroup(url: string) {
    if (!!this.navTabGroup) {
      this.navTabGroup.selectedIndex = NAVS.findIndex((v) => url === v);
    }
  }

  onOpenStart(opened: boolean) {
    this.store.dispatch(
      AppActions.openedLiftSideNav({
        opened
      })
    );
  }
}
