import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import { Component, OnInit, OnDestroy, Inject } from '@angular/core';

import * as detectBrowser from 'detect-browser';

import { Store } from '@ngrx/store';

import { NativeService, WindowState, NativeType } from '@ucap/native';

import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';

@Component({
  selector: 'app-layouts-top-bar',
  templateUrl: './top-bar.component.html',
  styleUrls: ['./top-bar.component.scss']
})
export class TopBarComponent implements OnInit, OnDestroy {
  platform = 'win32';
  windowState: WindowState;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private store: Store<any>
  ) {
    this.nativeService.platform_nativeType().then((type) => {
      switch (type) {
        case NativeType.Browser:
          this.platform = 'browser';
          break;
        case NativeType.Electron:
          {
            const info = detectBrowser.detect();
            if (info.os.startsWith('Windows')) {
              this.platform = 'win32';
            } else if (info.os.startsWith('Mac OS')) {
              this.platform = 'darwin';
            } else if (info.os.startsWith('Linux')) {
              this.platform = 'linux';
            } else {
            }
          }
          break;

        default:
          break;
      }
    });
  }

  ngOnInit() {
    this.nativeService
      .window_onState$()
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((windowState) => {
        this.windowState = windowState;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClosed() {
    this.nativeService.window_close();
  }

  onMaximized(altKey: boolean) {
    this.nativeService.window_maximize(altKey);
  }

  onMinimized() {
    this.nativeService.window_minimize();
  }
}
