import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef
} from '@angular/core';
import { ActivatedRoute, Params } from '@angular/router';

import { Subject } from 'rxjs';
import { takeUntil, withLatestFrom } from 'rxjs/operators';
import { QueryParams } from '../types/params.type';
import { Store, select } from '@ngrx/store';

import { UserInfoSS } from '@ucap/domain-organization';

import { UserSelector } from '@ucap/ng-store-organization';
import { AppOrganizationService } from '@app/services/app-organization.service';

import { ConfigurationSelector } from '@ucap/ng-store-authentication';

import {
  ProfileDialogComponent,
  ProfileDialogData,
  ProfileDialogResult
} from '@app/sections/organization/dialogs/profile.dialog.component';
import { MatDialog } from '@angular/material/dialog';
import { VersionInfo } from '@ucap/domain-authentication';

@Component({
  selector: 'app-pages-call-history',
  templateUrl: './call-history.page.component.html',
  styleUrls: ['./call-history.page.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class CallHistoryPageComponent implements OnInit, OnDestroy {
  isMe: boolean;
  userSeq: string;
  userInfo: UserInfoSS;

  versionInfo: VersionInfo;

  private ngOnDestroySubject: Subject<void> = new Subject();
  constructor(
    private dialog: MatDialog,
    private store: Store<any>,
    private appOrganizationService: AppOrganizationService,
    private activatedRoute: ActivatedRoute,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {
    this.activatedRoute.queryParams
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        withLatestFrom(this.store.pipe(select(UserSelector.user)))
      )
      .subscribe(([params, user]) => {
        const seqParam = params[QueryParams.ID];

        // initializing by userSeq Change.
        if (this.userSeq !== seqParam) {
        }

        // setting userSeq.
        this.userSeq = !!seqParam
          ? seqParam
          : !!user
          ? String(user.info.seq)
          : undefined;

        if (!!user && this.userSeq === String(user.info.seq)) {
          this.isMe = true;
        } else {
          this.isMe = false;
        }

        if (!!this.userSeq) {
          const self = this;
          this.appOrganizationService
            .getUserInfo({ userSeq: this.userSeq, user })
            .then((result) => {
              console.log('getUserInfo : ', result);
              self.userInfo = result.userInfo;
            })
            .catch((e) => {
              self.userInfo = undefined;
            })
            .finally(() => {
              this.changeDetectorRef.markForCheck();
            });
        } else {
          this.userInfo = undefined;
          this.changeDetectorRef.markForCheck();
        }
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onOpenProfile(userInfo: UserInfoSS): void {
    if (!!userInfo) {
      this.dialog.open<
        ProfileDialogComponent,
        ProfileDialogData,
        ProfileDialogResult
      >(ProfileDialogComponent, {
        panelClass: 'mid-create-dialog',
        data: {
          userSeq: userInfo.seq
        }
      });
    }
  }
}
