import { Subject, BehaviorSubject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import { Component, ViewChild, OnInit, OnDestroy } from '@angular/core';
import { ActivatedRoute, Params } from '@angular/router';

import { Store } from '@ngrx/store';

import { MatDrawer } from '@angular/material/sidenav';

import { ChattingActions } from '@ucap/ng-store-chat';

import { MessageSectionComponent } from '@app/sections/chat/components/message.section.component';
import {
  FormSectionComponent,
  SelectorType
} from '@app/sections/chat/components/form.section.component';

import { QueryParams } from '../types/params.type';
import { ChatDrawType } from '../types/chat-draw.type';
import { DrawInfo } from '../models/draw-info';
import { SearchInfo } from '../models/search-info';

@Component({
  selector: 'app-pages-chat-room',
  templateUrl: './chat-room.page.component.html',
  styleUrls: ['./chat-room.page.component.scss']
})
export class ChatRoomPageComponent implements OnInit, OnDestroy {
  isChatSearch = false;
  searchObj: SearchInfo = {
    isShowSearch: false,
    searchWord: ''
  };

  roomId: string;
  translationSimpleview = false;

  drawerType: ChatDrawType | null;
  returnDrawerType: ChatDrawType | null;
  eventSendTriggerSubject: BehaviorSubject<any> = new BehaviorSubject<any>(0);

  @ViewChild('chatMessageSections', { static: false })
  chatMessageSections: MessageSectionComponent;

  @ViewChild('chatRightDrawer', { static: false })
  chatRightDrawer: MatDrawer;

  @ViewChild('chatForm', { static: false })
  chatForm: FormSectionComponent;

  ChatDrawType = ChatDrawType;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private activatedRoute: ActivatedRoute
  ) {}

  ngOnInit(): void {
    this.activatedRoute.queryParams
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((params: Params) => {
        const seqParam = params[QueryParams.ROOM_ID];

        // initializing by roomId Change.
        if (this.roomId !== seqParam) {
          // close Right Drawer.
          if (!!this.chatRightDrawer) {
            this.chatRightDrawer.close();
          }

          // close Chat Search area.
          this.isChatSearch = false;
          this.searchObj = {
            isShowSearch: false,
            searchWord: ''
          };
        }

        // setting roomId.
        this.roomId = !!seqParam ? seqParam : undefined;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    if (!!this.eventSendTriggerSubject) {
      this.eventSendTriggerSubject.complete();
    }

    // this.store.dispatch(RoomActions.clearSelectedRoom({ roomId: this.roomId }));
    this.store.dispatch(ChattingActions.clearActiveRoomId({}));
  }

  /** About Form Selector */
  onOpenFormSelector() {
    if (!!this.chatMessageSections) {
      this.chatMessageSections.refreshAndScrollToBottom();
    }
  }

  /** About  Right drawer */
  onRightDrawerToggle(type: DrawInfo | null): void {
    this.drawerType = type.chatDrawType;
    this.returnDrawerType = !!type.returnDrawType ? type.returnDrawType : null;
    this.chatRightDrawer.open();
  }
  onRightDrawerClose(): void {
    this.drawerType = null;
    this.returnDrawerType = null;
    this.chatRightDrawer.close();
  }

  /** About File Drag & Drop */
  onFileSelected(fileList: FileList) {
    if (!!this.chatForm) {
      this.chatForm.onDragAndDropFileupload(fileList); // direct upload
      // this.chatForm.onDragAndDropStandbyFileupload(fileList); // standby and confirm upload.(complate not yet)
    }
  }
  onFileDragEnter(event: DataTransferItemList) {
    // if (!!this.chatForm) {
    //   this.chatForm.onOpenSelector(SelectorType.FILEUPLOAD);
    // }
  }
  onFileDragOver(event: DragEvent) {}
  onFileDragLeave(event: DragEvent) {
    // if (!!this.chatForm) {
    //   this.chatForm.clearSelector();
    // }
  }

  /** About Chat search. */
  onChatSearch(search: SearchInfo) {
    this.searchObj = search;
  }
}
