import { Subject, combineLatest } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectorRef,
  ChangeDetectionStrategy
} from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';

import { Store, select } from '@ngrx/store';

import { DeptInfo } from '@ucap/domain-organization';

import { LogService } from '@ucap/ng-logger';

import { DepartmentSelector, UserSelector } from '@ucap/ng-store-organization';
import { LoginSelector } from '@ucap/ng-store-authentication';

import { AppUiService } from '@app/services/app-ui.service';
import { AppSelector } from '@app/store/state';
import { WindowSizeMode } from '@app/types/window-size-mode.type';

import { environment } from '@environments';

import { QueryParams } from '../types/params.type';

@Component({
  selector: 'app-pages-ogranization-sidenav',
  templateUrl: './sidenav.page.component.html',
  styleUrls: ['./sidenav.page.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SidenavPageComponent implements OnInit, OnDestroy {
  initialExpanded: number;
  displayRoot = false;
  displayRootDept: DeptInfo;
  windowSizeMode: WindowSizeMode;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private router: Router,
    private activatedRoute: ActivatedRoute,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private appUiService: AppUiService,
    private logService: LogService
  ) {
    this.logService.info('app-pages-ogranization-sidenav');
  }

  ngOnInit(): void {
    // this.activatedRoute.queryParams
    //   .pipe(takeUntil(this.ngOnDestroySubject))
    //   .subscribe((params) => {
    //     if (!!params) {
    //       const deptSeq = params[QueryParams.DEPT_SEQ];
    //       if (!!deptSeq) {
    //         this.initialExpanded = Number(deptSeq);
    //       }
    //     }
    //   });

    combineLatest([
      this.activatedRoute.queryParams,
      this.store.pipe(select(UserSelector.user))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([params, user]) => {
        let existParams = false;
        if (!!params) {
          const deptSeq = params[QueryParams.DEPT_SEQ];
          if (!!deptSeq) {
            existParams = true;
            this.initialExpanded = Number(deptSeq);
          }
        }

        if (!existParams) {
          this.initialExpanded = user.departmentCode;
        }
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(DepartmentSelector.departmentInfoList)
      )
      .subscribe((deptInfoList) => {
        if (!environment.productConfig.organization.displayRoot) {
          if (!!deptInfoList && deptInfoList.length > 0) {
            this.displayRootDept = deptInfoList.find(
              (item) => item.type === 'R'
            );
          }
        }
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AppSelector.windowSizeMode)
      )
      .subscribe((windowSizeMode) => {
        this.windowSizeMode = windowSizeMode;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClickedTree(node: DeptInfo) {
    const queryParams: Params = {};
    queryParams[QueryParams.DEPT_SEQ] = String(node.seq);
    this.router.navigate(
      [
        'organization',
        {
          outlets: { content: 'index' }
        }
      ],
      {
        queryParams
      }
    );
    this.appUiService.closeLeftSidenavOnNarrowMode();
  }
}
