import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input
} from '@angular/core';

import { LogService } from '@ucap/ng-logger';

import { SearchData } from '@app/ucap/organization/models/search-data';
import { Store, select } from '@ngrx/store';
import { takeUntil } from 'rxjs/operators';
import { CallSelector } from '@ucap/ng-store-call';
import {
  CallHistory,
  CallHistorySearchType,
  CallResultType
} from '@ucap/domain-call';

@Component({
  selector: 'app-sections-call-list',
  templateUrl: './list.section.component.html',
  styleUrls: ['./list.section.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ListSectionComponent implements OnInit, OnDestroy {
  @Input()
  set searchObj(obj: SearchData) {
    this._getFiltered();
  }
  get searchObj(): SearchData {
    return this._searchObj;
  }
  // tslint:disable-next-line: variable-name
  _searchObj: SearchData;

  callHistory: CallHistory[];
  filteredCallHistory: CallHistory[];

  searchType = CallHistorySearchType.All;
  CallHistorySearchType = CallHistorySearchType;

  private ngOnDestroySubject: Subject<void> = new Subject();
  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(CallSelector.callHistory)
      )
      .subscribe((callHistory) => {
        this.callHistory = callHistory;
        this._getFiltered();
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  filteringByTypeClass(searchType: CallHistorySearchType): string {
    return this.searchType === searchType ? 'cdk-active' : '';
  }
  filteringByType(searchType: CallHistorySearchType) {
    this.searchType = searchType;
    this._getFiltered();
  }

  private _getFiltered() {
    if (!!this.callHistory && this.callHistory.length > 0) {
      switch (this.searchType) {
        case CallHistorySearchType.Send:
          this.filteredCallHistory = this.callHistory.filter(
            (item) => !!item.sendYn
          );
          break;
        case CallHistorySearchType.Receive:
          this.filteredCallHistory = this.callHistory.filter(
            (item) => !item.sendYn
          );
          break;
        case CallHistorySearchType.Missed:
          this.filteredCallHistory = this.callHistory.filter(
            (item) => item.callResult === CallResultType.Away
          );
          break;
        case CallHistorySearchType.UnAnswered:
          this.filteredCallHistory = this.callHistory.filter(
            (item) => item.callResult === CallResultType.Missed
          );
          break;
        default:
          // case CallHistorySearchType.All:
          this.filteredCallHistory = this.callHistory;
          break;
      }
    } else {
      this.filteredCallHistory = [];
    }
    this.changeDetectorRef.markForCheck();
  }
}
