import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject
} from '@angular/core';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { Subject } from 'rxjs';
import { Store, select } from '@ngrx/store';
import { takeUntil } from 'rxjs/operators';

import { User } from '@ucap/domain-organization';
import { UserSelector } from '@ucap/ng-store-organization';
import { LoginSelector } from '@ucap/ng-store-authentication';
import { AppCallService } from '@app/services/app-call.service';
import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { LoginSession, LoginInfo } from '@ucap/domain-authentication';

export interface DialpadDialogData {}
export interface DialpadDialogResult {}

@Component({
  selector: 'app-dialog-call-dialpad',
  templateUrl: './dialpad.dialog.component.html',
  styleUrls: ['./dialpad.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class DialpadDialogComponent implements OnInit, OnDestroy {
  loginSession: LoginSession;
  loginInfo: LoginInfo;
  user: User;

  private ngOnDestroySubject: Subject<void> = new Subject();
  constructor(
    private store: Store<any>,
    public dialogRef: MatDialogRef<DialpadDialogData, DialpadDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: DialpadDialogData,
    private appCallService: AppCallService,
    private appAuthenticationService: AppAuthenticationService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {
    this.loginSession = this.appAuthenticationService.getLoginSession();
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(LoginSelector.loginInfo))
      .subscribe((loginInfo) => {
        this.loginInfo = loginInfo;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onSendCall(calleeNumber: string) {
    this.appCallService.sendClicktocall(
      this.loginInfo,
      this.user,
      this.loginSession.deviceType,
      calleeNumber
    );
    this.dialogRef.close();
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }
}
