import { Subject, combineLatest } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject
} from '@angular/core';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { I18nService } from '@ucap/ng-i18n';

import { Store, select } from '@ngrx/store';
import { takeUntil } from 'rxjs/operators';
import { AppChatService } from '@app/services/app-chat.service';
import {
  TranslatePipe as OrganizationTranslate,
  TranslateService
} from '@ucap/ng-ui-organization';

export interface ClipboardDialogData {
  content: {
    text?: string;
    rtf?: string;
    html?: string;
    image?: Buffer;
    imageDataUrl?: string;
  };
}
export interface ClipboardDialogResult {}

@Component({
  selector: 'app-dialog-chat-clipboard',
  templateUrl: './clipboard.dialog.component.html',
  styleUrls: ['./clipboard.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ClipboardDialogComponent implements OnInit, OnDestroy {
  constructor(
    public dialogRef: MatDialogRef<ClipboardDialogData, ClipboardDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: ClipboardDialogData,
    private store: Store<any>,
    private i18nService: I18nService,
    public dialog: MatDialog,
    private changeDetectorRef: ChangeDetectorRef,
    private appChatService: AppChatService,
    private translateService: TranslateService
  ) {
    // language setting
    this.translateService.setDefaultLang(this.i18nService.currentLng);
    this.translateService.use(this.i18nService.currentLng);
  }

  private ngOnDestroySubject: Subject<void> = new Subject();

  ngOnInit(): void {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close({ choice: false });
  }

  onCancel() {
    this.dialogRef.close({ choice: false });
  }
  onConfirm() {}
}
