import { Subject } from 'rxjs';
import { takeUntil, take, map } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { User, UserInfoSS } from '@ucap/domain-organization';

import { QueryProtocolService } from '@ucap/ng-protocol-query';

import { UserSelector } from '@ucap/ng-store-organization';
import { AuthorizationSelector } from '@ucap/ng-store-authentication';
import { UserPermission } from '@ucap/domain-authorization';

@Component({
  selector: 'app-sections-group-profile',
  templateUrl: './profile.section.component.html',
  styleUrls: ['./profile.section.component.scss'],

  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileSectionComponent implements OnInit, OnDestroy {
  @Input()
  set userSeq(seq: string) {
    this._userSeq = seq;
    this.getUserInfo(seq);
  }
  get userSeq(): string {
    return this._userSeq;
  }
  // tslint:disable-next-line: variable-name
  _userSeq: string;

  user: User;
  userPermission: UserPermission;
  isMe = false;

  profileUserInfo: UserInfoSS;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private queryProtocolService: QueryProtocolService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;

        if (!!user) {
          const seq = !!this.userSeq ? this.userSeq : String(user.info.seq);
          this.getUserInfo(seq);
        }

        if (!!this.userSeq && this.userSeq === String(user.info.seq)) {
          this.isMe = true;
        } else {
          this.isMe = false;
        }
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AuthorizationSelector.userPermission)
      )
      .subscribe((userPermission) => {
        this.userPermission = userPermission;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  getUserInfo(userSeq: string | number): void {
    if (!!userSeq && !!this.user && !!this.user.info) {
      this.queryProtocolService
        .dataUser({
          divCd: 'OPENPROF',
          seq: Number(userSeq + ''),
          senderCompanyCode: this.user.info.companyCode,
          senderEmployeeType: this.user.info.employeeType
        })
        .pipe(
          take(1),
          map((res) => {
            if (String(this.user.info.seq) === String(userSeq)) {
              this.isMe = true;
            } else {
              this.isMe = false;
            }

            this.profileUserInfo = res.userInfo;
            this.changeDetectorRef.detectChanges();
          })
        )
        .subscribe();
    }
  }

  onToggleBuddy(param: { userInfo: UserInfoSS; isBuddy: boolean }) {
    console.log('onToggleBuddy : ', param);
  }
  onToggleFavorite(param: { userInfo: UserInfoSS; isFavorite: boolean }) {
    console.log('onToggleFavorite : ', param);
  }

  onOpenChat(userInfo: UserInfoSS) {
    console.log('onOpenChat : ', userInfo);
  }

  onSendCall(calleeNum: string) {
    console.log('onSendCall : ', calleeNum);
  }

  onSendSms(calleenum: string) {
    console.log('onSendSms : ', calleenum);
  }

  onCreateConference(userSeq: number) {
    console.log('onCreateConference : ', userSeq);
  }

  onSendMessage(userInfo: UserInfoSS) {
    console.log('onSendMessage : ', userInfo);
  }
}
