import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Output,
  EventEmitter,
  Input,
  ViewChild
} from '@angular/core';

import { MatCheckboxChange } from '@angular/material/checkbox';

import { Store } from '@ngrx/store';

import { UserInfo, DeptInfo, UserInfoSS } from '@ucap/domain-organization';
import { GroupInfoDetail } from '@ucap/domain-group';

import { SearchData } from '@app/ucap/organization/models/search-data';
import { ProfileListComponent as AppGroupProfileListComponent } from '@app/ucap/group/components/profile-list.component';
import { SelectUserDialogType, UserInfoTypes } from '@app/types';
import { ProfileNavigationListComponent } from '@app/ucap/organization/components/profile-navigation-list.component';
import { Expansion02Component as AppExpansion02Component } from '@app/ucap/group/components/expansion-02.component';

@Component({
  selector: 'app-group-select-user',
  templateUrl: './select-user.section.component.html',
  styleUrls: ['./select-user.section.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SelectUserSectionComponent
  implements OnInit, OnDestroy, AfterViewInit {
  @Input()
  isDialog = false;

  @Input()
  checkable = false;

  @Input()
  selectedUserList: UserInfoTypes[];

  @Input()
  fixedUserList: UserInfoTypes[];

  @Input()
  displayRootDepartment = false;

  @Input()
  selectedGroupHeader: GroupInfoDetail;

  @Input()
  filteredTimerChatAuth = false;

  @Output()
  toggleCheckUser: EventEmitter<
    {
      checked: boolean;
      userInfo: UserInfoTypes;
    }[]
  > = new EventEmitter();

  @Output()
  toggleCheckGroup: EventEmitter<{
    isChecked: boolean;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
  }> = new EventEmitter();

  @ViewChild('groupDialogExpasion', { static: false })
  groupDialogExpasion: AppExpansion02Component;

  @ViewChild('groupProfileList', { static: false })
  groupProfileList: AppGroupProfileListComponent;

  @ViewChild('profileNavigationList', { static: false })
  profileNavigationList: ProfileNavigationListComponent;

  set companySearchData(searchData: SearchData) {
    if (!!searchData && searchData.searchWord !== '') {
      this._companySearchData = { ...searchData, bySearch: true };
    } else {
      this._companySearchData = { ...searchData, bySearch: false };
    }

    this.onChangedCompanySearch();
  }
  get companySearchData() {
    return this._companySearchData;
  }
  // tslint:disable-next-line: variable-name
  _companySearchData: SearchData;

  isBackspaceCanceled = true;
  isSearch = false;

  isAllCheck: boolean;

  searchedUserInfos: UserInfoTypes[] = [];
  searchedUserInfosLength: number;

  breadcrumbs: DeptInfo[];
  departmentInfoList: DeptInfo[] = [];
  sameLevelDeptInfoList: DeptInfo[] = [];
  memberSearchData: SearchData;

  SelectUserDialogType = SelectUserDialogType;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private changeDetectorRef: ChangeDetectorRef,
    private store: Store<any>
  ) {}

  ngOnInit(): void {
    if (!!this.selectedUserList && this.selectedUserList.length > 0) {
      this.selectedUserList.map((user: UserInfoTypes) =>
        this.onToggleCheckUser([{ checked: true, userInfo: user }])
      );
    }
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  ngAfterViewInit(): void {}

  psUpdate() {
    this.groupDialogExpasion.psUpdate();
  }

  onToggleCheckGroup(params: {
    isChecked: boolean;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
  }) {
    this.toggleCheckGroup.emit(params);
  }

  onToggleCheckUser(datas: { checked: boolean; userInfo: UserInfoTypes }[]) {
    datas.forEach((data) => {
      data.userInfo = { ...data.userInfo, seq: Number(data.userInfo.seq) };
    });
    this.toggleCheckUser.emit(datas);
  }

  onChangedCompanySearch() {
    if (!!this.companySearchData && this.companySearchData.bySearch) {
      this.isSearch = true;
    }
    this.changeDetectorRef.markForCheck();
  }

  onCanceled() {
    this.isSearch = false;
    this.companySearchData = { ...this.companySearchData, searchWord: '' };
  }

  onIsAllCheck(checked: boolean): void {
    this.isAllCheck = checked;
    this.changeDetectorRef.detectChanges();
  }

  onSearched(userInfo: UserInfoSS[]) {
    this.searchedUserInfosLength = userInfo.length;
  }
  getAllCheckedItem(): boolean {
    if (!!this.groupProfileList) {
      const targetList = this.groupProfileList.userInfos;
      if (
        !targetList ||
        targetList.length === 0 ||
        targetList.filter(
          (item) =>
            !(
              this.selectedUserList.filter((info) => info.seq === item.seq)
                .length > 0
            )
        ).length > 0
      ) {
        return false;
      } else {
        return true;
      }
    }

    return false;
  }

  onAllChecked(event: MatCheckboxChange): void {
    if (event.checked) {
      this.groupProfileList.checkAll();
    } else {
      this.groupProfileList.uncheckAll();
    }
  }

  onChangedCheckProfileList(
    datas: { checked: boolean; userInfo: UserInfoTypes }[]
  ) {
    if (!datas || 0 === datas.length) {
      return;
    }
  }
}
