import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  Input
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { UserInfo, UserInfoSS } from '@ucap/domain-organization';
import { GroupInfoDetail } from '@ucap/domain-group';

import { I18nService } from '@ucap/ng-i18n';

import { SelectUserDialogType, GroupManageType } from '@app/types';

import { UserInfoTypes } from '@app/types';

import { takeUntil } from 'rxjs/operators';
import { GroupSelector } from '@ucap/ng-store-group';
import { AppGroupService } from '@app/services/app-group.service';
// import { LogService } from '@ucap/logger';
export interface EditUserDialogData {
  title: string;
  type: GroupManageType;
  group?: GroupInfoDetail;
  userInfos: UserInfoTypes[];
}
export interface EditUserDialogResult {
  type: GroupManageType;
  groupName: string;
  group: GroupInfoDetail;
  selelctUserList?: UserInfoTypes[];
  selectGroupList?: GroupInfoDetail[];
}

@Component({
  selector: 'app-dialog-edit-user',
  templateUrl: './edit-user.dialog.component.html',
  styleUrls: ['./edit-user.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class EditUserDialogComponent implements OnInit, OnDestroy {
  constructor(
    public dialogRef: MatDialogRef<EditUserDialogData, EditUserDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: EditUserDialogData,
    private appGroupService: AppGroupService,
    // private logService: LogService,
    private changeDetectorRef: ChangeDetectorRef,
    private store: Store<any>,
    private i18nService: I18nService,
    public dialog: MatDialog
  ) {}

  private ngOnDestroySubject: Subject<void> = new Subject();

  selectedUserList: UserInfoTypes[] = [];
  selectedGroupList: GroupInfoDetail[] = [];
  groupName: string;
  SelectUserDialogType = SelectUserDialogType;
  groupList: GroupInfoDetail[] = [];

  ngOnInit(): void {
    this.selectedUserList = [...this.data.userInfos];

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(GroupSelector.groups))
      .subscribe((groups) => {
        this.groupList = groups;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }

  onConfirm() {
    this.dialogRef.close({
      type: this.data.type,
      groupName: this.groupName,
      group: this.data.group,
      selelctUserList: this.selectedUserList,
      selectGroupList: this.selectedGroupList
    });
  }

  onToggleCheckUser(
    datas: {
      checked: boolean;
      userInfo: UserInfoSS;
    }[]
  ) {
    const resList = this.appGroupService.getToggleCheckUser(
      datas,
      this.selectedUserList
    );

    if (!resList) {
      return;
    }

    this.selectedUserList = resList;
  }

  onToggleCheckGroup(data: { checked: boolean; group: GroupInfoDetail }) {
    this.selectedGroupList = this.appGroupService.getToggleCheckForGroupList(
      data,
      this.selectedGroupList
    );
  }

  onChangeGroupName(data: { invalid: boolean; groupName: string }) {
    this.groupName = data.groupName;
  }

  checkDisableBtn(): boolean {
    const errMsg = this.appGroupService.validateGroupName(
      this.groupName,
      this.groupList
    );

    if (
      !!errMsg &&
      this.groupName === '' &&
      !!this.selectedGroupList &&
      this.selectedGroupList.length > 0
    ) {
      return false;
    } else if (
      !!errMsg &&
      !!this.selectedGroupList &&
      this.selectedGroupList.length === 0
    ) {
      return true;
    } else if (!!errMsg && this.groupName !== '') {
      return true;
    }

    return false;
  }

  onRemovedProfileSelection(userInfo: UserInfo) {
    const resList = this.appGroupService.removedProfileSelection(
      userInfo,
      this.selectedUserList
    );
    if (!resList) {
      // this.logService.error('onRemovedProfileSelection', 'null');
    }

    this.selectedUserList = resList;
  }

  removableForSelection = (userInfo: UserInfo) => {
    return true;
  };

  colorForSelection = (userInfo: UserInfo) => {
    return 'accent';
  };
}
