import { Subject, of } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  ViewChild,
  ViewContainerRef
} from '@angular/core';

import { Store } from '@ngrx/store';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { UserInfo, UserInfoSS } from '@ucap/domain-organization';
import { GroupInfoDetail } from '@ucap/domain-group';

import { I18nService } from '@ucap/ng-i18n';

import { UserInfoTypes } from '@app/types';
import { LogService } from '@ucap/ng-logger';
import { SelectUserSectionComponent } from '../components/select-user.section.component';

import { AppGroupService } from '@app/services/app-group.service';

export interface ManageDialogData {
  title: string;
  groupBuddyList?: { group: GroupInfoDetail; buddyList: UserInfo[] };
}
export interface ManageDialogResult {
  choice: boolean;
  group?: GroupInfoDetail;
  selelctUserList?: UserInfoTypes[];
}

@Component({
  selector: 'app-dialog-group-manage',
  templateUrl: './manage.dialog.component.html',
  styleUrls: ['./manage.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ManageDialogComponent implements OnInit, OnDestroy, AfterViewInit {
  constructor(
    public dialogRef: MatDialogRef<ManageDialogData, ManageDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: ManageDialogData,
    private changeDetectorRef: ChangeDetectorRef,
    private i18nService: I18nService,
    public dialog: MatDialog,
    private logService: LogService,
    private appGroupService: AppGroupService
  ) {}

  @ViewChild('dialogContainer', { static: true, read: ViewContainerRef })
  dialogContainer: ViewContainerRef;

  @ViewChild('appGroupSelectUser', { static: false })
  appGroupSelectUser: SelectUserSectionComponent;

  private ngOnDestroySubject: Subject<void> = new Subject();

  selectedUserList: UserInfoTypes[] = [];
  selectedGroupList: GroupInfoDetail[] = [];

  ngOnInit(): void {
    this.selectedUserList = this.data.groupBuddyList.buddyList;
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  ngAfterViewInit() {
    this.psUpdate();
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }

  psUpdate() {
    this.appGroupSelectUser.psUpdate();
  }

  onToggleCheckGroup(params: {
    isChecked: boolean;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
  }) {
    const resList = this.appGroupService.getToggleCheckGroup(
      params,
      this.selectedUserList
    );

    if (!resList) {
      return;
    }
    this.selectedUserList = resList;
  }

  onToggleCheckUser(
    datas: {
      checked: boolean;
      userInfo: UserInfoSS;
    }[]
  ) {
    const resList = this.appGroupService.getToggleCheckUser(
      datas,
      this.selectedUserList
    );

    if (!resList) {
      return;
    }
    this.selectedUserList = resList;
  }
  onCancel(event: MouseEvent) {
    this.dialogRef.close({ choice: false });
  }
  onConfirm(event: MouseEvent) {
    this.dialogRef.close({
      choice: true,
      group: {
        ...this.data.groupBuddyList.group
      },
      selelctUserList: this.selectedUserList
    });
  }

  /** 개별 체크여부 */
  getCheckedUser(userInfo: UserInfoSS) {
    if (!!this.selectedUserList && this.selectedUserList.length > 0) {
      return (
        this.selectedUserList.filter(
          (item) => String(item.seq) === String(userInfo.seq)
        ).length > 0
      );
    }

    return false;
  }

  checkDisableBtn(): boolean {
    let findUser: UserInfoTypes[];
    if (!!this.selectedUserList && this.selectedUserList.length > 0) {
      findUser = this.selectedUserList.filter((user) => {
        return this.data.groupBuddyList.group.userSeqs.includes(
          String(user.seq)
        );
      });
    }

    if (!!findUser && findUser.length > 0) {
      return false;
    } else {
      return true;
    }
  }

  onToggleCheck(data: { checked: boolean; userInfo: UserInfoSS }) {
    this.onToggleCheckUser([data]);
  }

  onRemovedProfileSelection(userInfo: UserInfo) {
    const resUserList = this.appGroupService.removedProfileSelection(
      userInfo,
      this.selectedUserList
    );

    if (!resUserList) {
    }
    this.selectedUserList = resUserList;
  }

  removableForSelection = (userInfo: UserInfo) => {
    return true;
  };

  colorForSelection = (userInfo: UserInfo) => {
    return 'accent';
  };
}
