import { of } from 'rxjs';
import { take, map, finalize, catchError } from 'rxjs/operators';

import { Injectable, Inject, NgZone } from '@angular/core';

import { Store } from '@ngrx/store';

import { NativeService } from '@ucap/native';

import { I18nService } from '@ucap/ng-i18n';
import { CommonApiService } from '@ucap/ng-api-common';
import { LogService } from '@ucap/ng-logger';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { FileProtocolService } from '@ucap/ng-protocol-file';

import { AppAuthenticationService } from './app-authentication.service';
import { MatDialog } from '@angular/material/dialog';
import {
  DialpadDialogComponent,
  DialpadDialogData,
  DialpadDialogResult
} from '@app/sections/call/dialogs/dialpad.dialog.component';

import { User } from '@ucap/domain-organization';
import { DeviceType } from '@ucap/domain-common';
import { SendClicktocallRequest } from '@ucap/api-message';
import { MessageApiService } from '@ucap/ng-api-message';
import { LoginInfo } from '@ucap/domain-authentication';
import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui/core';

@Injectable({
  providedIn: 'root'
})
export class AppCallService {
  constructor(
    private dialog: MatDialog,
    private store: Store<any>,
    private i18nService: I18nService,
    private commonApiService: CommonApiService,
    private messageApiService: MessageApiService,
    private appAuthenticationService: AppAuthenticationService,
    private logService: LogService
  ) {}

  sendClicktocall(
    loginInfo: LoginInfo,
    user: User,
    deviceType: DeviceType,
    calleeNumber: string
  ) {
    if (!!calleeNumber || calleeNumber.trim().length === 0) {
      this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('call:label.error'),
          message: this.i18nService.t('call:errors.noExistTargetnumber')
        }
      });
      return;
    }

    const req: SendClicktocallRequest = {
      userSeq: String(user.info.seq),
      deviceType,
      tokenKey: loginInfo.tokenString,
      calleeNumber
    };

    this.messageApiService
      .sendClicktocall(req)
      .pipe(
        take(1),
        map((res) => {}),
        catchError((error) => of({ error }))
      )
      .subscribe();
  }
}
