import { of } from 'rxjs';
import { take, catchError, map } from 'rxjs/operators';

import { Injectable, Inject } from '@angular/core';

import { MatDialog } from '@angular/material/dialog';

import { Store } from '@ngrx/store';

import {
  UserInfo,
  UserInfoF,
  UserInfoSS,
  UserInfoDN,
  UserInfoUpdateType
} from '@ucap/domain-organization';
import { GroupInfoDetail } from '@ucap/domain-group';

import { NativeService } from '@ucap/native';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';

import { UserActions } from '@ucap/ng-store-organization';

import { GroupActions, BuddyActions } from '@ucap/ng-store-group';

import {
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult,
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui/core';

import { UserInfoTypes, GroupManageType } from '@app/types';
import { AppService } from './app.service';

@Injectable({
  providedIn: 'root'
})
export class AppGroupService {
  constructor(
    private dialog: MatDialog,
    private store: Store<any>,
    private i18nService: I18nService,
    protected logService: LogService,
    private appService: AppService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService
  ) {}

  addMemberToGroup(
    targetGroup: GroupInfoDetail,
    userInfoList: UserInfoTypes[]
  ) {
    const targetUserSeqs: string[] = [...targetGroup.userSeqs];

    userInfoList.map((userInfo) => {
      const find = targetUserSeqs.indexOf(String(userInfo.seq));
      if (find < 0) {
        targetUserSeqs.push(String(userInfo.seq));
      }
    });

    this._updateMember(targetGroup, targetUserSeqs);
  }

  updateMemberToGroup(
    targetGroup: GroupInfoDetail,
    userInfoList: UserInfoTypes[]
  ) {
    const targetUserSeqs = userInfoList.map((userInfo) => String(userInfo.seq));
    this._updateMember(targetGroup, targetUserSeqs);
  }
  moveMemberToGroup(
    fromGroup: GroupInfoDetail,
    toGroup: GroupInfoDetail,
    userInfoList: UserInfoTypes[]
  ) {
    const targetUserSeqs: string[] = [];

    userInfoList.map((userInfo) => {
      targetUserSeqs.push(String(userInfo.seq));
    });

    this.store.dispatch(
      GroupActions.moveMember({
        fromGroup,
        toGroup,
        targetUserSeq: targetUserSeqs
      })
    );
  }
  copyMemberToGroup(
    targetGroup: GroupInfoDetail,
    userInfoList: UserInfoTypes[]
  ) {
    const targetUserSeqs: string[] = [...targetGroup.userSeqs];

    userInfoList.map((userInfo) => {
      const find = targetUserSeqs.indexOf(String(userInfo.seq));
      if (find < 0) {
        targetUserSeqs.push(String(userInfo.seq));
      }
    });

    this._updateMember(targetGroup, targetUserSeqs);
  }

  removeMember(group: GroupInfoDetail, selelctUserList: UserInfoTypes[]) {
    const trgtUserSeq: string[] = [];
    const selectSeq: string[] = [];

    selelctUserList.map((user) => selectSeq.push(String(user.seq)));

    group.userSeqs.filter((buddySeq) => {
      if (!trgtUserSeq.includes(buddySeq) && !selectSeq.includes(buddySeq)) {
        trgtUserSeq.push(buddySeq);
      }
    });

    this._updateMember(group, trgtUserSeq);
  }

  removeMemberToGroup(
    contens: string,
    userInfoList: UserInfoTypes[],
    group: GroupInfoDetail
  ): Promise<void> {
    return new Promise<void>((resolve, rejects) => {
      try {
        const dialogRef = this.dialog.open<
          ConfirmDialogComponent,
          ConfirmDialogData,
          ConfirmDialogResult
        >(ConfirmDialogComponent, {
          panelClass: 'min-create-dialog',
          data: {
            title: this.i18nService.t('group:dialog.title.removeBuddy'),
            html: contens
          }
        });
        dialogRef
          .afterClosed()
          .pipe(
            take(1),
            map((result) => {
              if (!!result && result.choice) {
                this.removeMember(group, userInfoList);
                resolve();
              }
            }),
            catchError((err) => {
              return of(err);
            })
          )
          .subscribe();
      } catch (err) {
        rejects();
      }
    });
  }

  createGroup(groupName: string, userInfoList: UserInfoTypes[]): Promise<void> {
    return new Promise<void>((resolve, rejects) => {
      try {
        const targetUserSeqs: string[] = [];
        userInfoList.map((user) => targetUserSeqs.push(String(user.seq)));
        this.store.dispatch(
          GroupActions.create({
            groupName,
            targetUserSeqs
          })
        );
        resolve();
      } catch (error) {
        rejects(error);
      }
    });
  }
  removeGroup(groupBuddyList: {
    group: GroupInfoDetail;
    buddyList: UserInfo[];
  }): Promise<void> {
    return new Promise<void>((resolve, rejects) => {
      try {
        const dialogRef = this.dialog.open<
          ConfirmDialogComponent,
          ConfirmDialogData,
          ConfirmDialogResult
        >(ConfirmDialogComponent, {
          panelClass: 'min-create-dialog',
          data: {
            title: this.i18nService.t('group:dialog.title.removeGroup'),
            html: this.i18nService.t('group:dialog.removeGroupConfirm', {
              targetGroup: `${groupBuddyList.group.name}`
            })
          }
        });
        dialogRef
          .afterClosed()
          .pipe(take(1))
          .subscribe((result) => {
            if (!!result && !!result.choice) {
              this.store.dispatch(
                GroupActions.del({ group: groupBuddyList.group })
              );

              resolve();
            }
          });
      } catch (error) {
        rejects(error);
      }
    });
  }

  updateGroupName(group: GroupInfoDetail, groupName: string): Promise<void> {
    return new Promise<void>((resolve, rejects) => {
      try {
        this.store.dispatch(
          GroupActions.update({
            req: {
              groupSeq: group.seq,
              groupName,
              userSeqs: group.userSeqs
            }
          })
        );
        resolve();
      } catch (error) {
        rejects(error);
      }
    });
  }

  addBuddy(datas: {
    type: GroupManageType;
    groupName?: string;
    group?: GroupInfoDetail;
    selelctUserList?: UserInfoTypes[];
    selectGroupList?: GroupInfoDetail[];
  }): Promise<boolean> {
    return new Promise<boolean>((resolve, rejects) => {
      try {
        if (!!datas && !!datas.groupName && datas.groupName.trim() !== '') {
          this.createGroup(datas.groupName, datas.selelctUserList);
        }

        if (
          !!datas &&
          !!datas.selectGroupList &&
          datas.selectGroupList.length > 0
        ) {
          datas.selectGroupList.forEach((g) => {
            this.addMemberToGroup(g, datas.selelctUserList);
          });
        }
        resolve(false);
      } catch (error) {
        rejects(error);
      }
    });
  }

  validateGroupName(groupName: string, groupList: GroupInfoDetail[]): string {
    // const forbidden = /[\{\}\[\]\/?.;:|\)*~`!^+<>@\#$%&\\\=\(\'\"]/g.test(
    //   groupName
    // );

    if (!groupName || !groupList) {
      return;
    }

    const forbidden = new RegExp(
      '[\\{\\}\\[\\]\\/?.;:|\\)*~`!^+<>@\\#$%&\\\\\\=\\(\\\'\\"|\\u00a1-' +
        '\\u00ac|\\u00ae-\\u00ff|\\u0100-\\u017f|\\u0180-\\u024f|\\' +
        'u1e02-\\u1ef3|\\u0259-\\u0292|\\u02b0-\\u02ff|\\u0370-\\u' +
        '03ff|\\u1f00-\\u1fff|\\u0400-\\u04ff|\\u2013-\\u204a|\\u2' +
        '010-\\u206f|\\u2070-\\u209f|\\u20a0-\\u20bf|\\u2100-\\u21' +
        '4f|\\u2150-\\u218f|\\u2190-\\u21ff|\\u2200-\\u22ff|\\u230' +
        '0-\\u23ff|\\u2460-\\u24ff|\\u2500-\\u257f|\\u2580-\\u259f' +
        '|\\u25a0-\\u25ff|\\u2600-\\u26ff|\\u3041-\\u309f|\\u30a0-' +
        '\\u30ff|\\u3100-\\u312f|\\u2e80-\\u2eff|\\u31c0-\\u31ef|\\' +
        'u3000-\\u303f|\\u3200-\\u32ff|\\uff00-\\uffef|\\u3190-\\u' +
        '319f|\\u2f00-\\u2fdf|\\u31f0-\\u31ff|\\ua4d0-\\ua4ff|\\ua' +
        '700-\\ua71f|\\ufe50-\\ufe6f|\\ufe10-\\ufe1f|\\ua000-\\ua4' +
        '8c|\\ua490-\\ua4cf|\\u33c2-\\u33d8]',
      'g'
    ).test(groupName);

    const ban = this.appService.bannedGroupNames.filter(
      (name) => name.toLowerCase() === groupName.trim().toLowerCase()
    )[0];

    if (forbidden) {
      return this.i18nService.t('group:error.useOnlyForSpecialCharacter', {
        specialCharacter: '-,'
      });
    } else if (groupName.trim().localeCompare('') === 0) {
      return this.i18nService.t('group:error.requireInput');
    } else if (!!ban && ban !== '') {
      return this.i18nService.t('group:error.bannedWords', {
        bannedWords: this.appService.bannedGroupNames.join(',')
      });
    } else if (
      -1 < groupList.findIndex((g) => g.name.trim() === groupName.trim())
    ) {
      return this.i18nService.t('group:error.sameNameExist');
    }
    return null;
  }

  updateBuddy(userInfo: UserInfoTypes, isFavorite: boolean): Promise<void> {
    return new Promise<void>((resolve, rejects) => {
      try {
        this.store.dispatch(
          BuddyActions.update({
            req: { seq: Number(userInfo.seq), isFavorit: isFavorite }
          })
        );
        resolve();
      } catch (error) {
        rejects(error);
      }
    });
  }
  removeBuddy(userInfo: UserInfoTypes): Promise<void> {
    return new Promise<void>((resolve, rejects) => {
      try {
        const tempUser = userInfo as
          | UserInfo
          | UserInfoSS
          | UserInfoF
          | UserInfoDN;

        const dialogRef = this.dialog.open<
          ConfirmDialogComponent,
          ConfirmDialogData,
          ConfirmDialogResult
        >(ConfirmDialogComponent, {
          panelClass: 'min-create-dialog',
          data: {
            title: this.i18nService.t('group:dialog.title.removeBuddy'),
            html: this.i18nService.t(
              'group:dialog.removeBuddyFromProfile',

              { target: `${tempUser.name} ${tempUser.grade}` }
            )
          }
        });

        dialogRef
          .afterClosed()
          .pipe(take(1))
          .subscribe((result) => {
            if (!!result && !!result.choice) {
              this.store.dispatch(
                BuddyActions.delAndClear({ seq: Number(userInfo.seq) })
              );

              resolve();
            }
          });
      } catch (error) {
        rejects(error);
      }
    });
  }

  checkBuddy(buddyList: UserInfoTypes[], userInfo: UserInfoTypes): boolean {
    const findBuddy = buddyList.filter(
      (b) => String(b.seq) === String(userInfo.seq)
    );

    if (!findBuddy || findBuddy.length === 0) {
      const result = this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('group:error.title.default'),
          message: '닉네임은 동료만 설정할 수 있습니다.'
        }
      });

      return false;
    }

    return true;
  }

  updateNickname(userInfo: UserInfoF, nickname: string) {
    let userSeq: number;
    try {
      userSeq = Number(userInfo.seq);
    } catch (error) {
      this.logService.error(error);
    }

    this.store.dispatch(
      BuddyActions.nickname({
        req: {
          userSeq,
          nickname
        }
      })
    );
  }

  updateIntro(info: string, type: UserInfoUpdateType) {
    if (!!info && info.trim().localeCompare('') === 0) {
      info = ' ';
    }
    this.store.dispatch(
      UserActions.modifyInfo({
        req: {
          type,
          info
        }
      })
    );
  }

  getToggleCheckForGroupList(
    data: {
      checked: boolean;
      group: GroupInfoDetail;
    },
    selectedGroupList: GroupInfoDetail[]
  ): GroupInfoDetail[] {
    const i = selectedGroupList.findIndex(
      (u) => String(u.seq) === String(data.group.seq)
    );

    if (data.checked) {
      if (-1 === i) {
        return [...selectedGroupList, data.group];
      }
    } else {
      if (-1 < i) {
        return selectedGroupList.filter(
          (u) => String(u.seq) !== String(data.group.seq)
        );
      }
    }
  }

  getToggleCheckGroup(
    data: {
      isChecked: boolean;
      groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
    },
    selectedUserList: UserInfoTypes[]
  ): UserInfoTypes[] {
    if (!data) {
      return null;
    }

    if (data.isChecked) {
      const tempList = [...selectedUserList];

      data.groupBuddyList.buddyList.forEach((item) => {
        if (
          selectedUserList.filter(
            (user) => String(user.seq) === String(item.seq)
          ).length === 0
        ) {
          tempList.push(item);
        }
      });

      return tempList;
    } else {
      return selectedUserList.filter(
        (item) =>
          data.groupBuddyList.buddyList.filter(
            (del) => String(del.seq) === String(item.seq)
          ).length === 0
      );
    }
  }

  getToggleCheckUser(
    datas: {
      checked: boolean;
      userInfo: UserInfoSS;
    }[],
    selectedUserList: UserInfoTypes[]
  ): UserInfoTypes[] {
    if (!datas || 0 === datas.length) {
      return null;
    }

    const pushs: UserInfoSS[] = [];
    const pops: UserInfoSS[] = [];

    datas.forEach((d) => {
      const i = selectedUserList.findIndex((u) => u.seq === d.userInfo.seq);
      if (d.checked) {
        if (-1 === i) {
          pushs.push(d.userInfo);
        }
      } else {
        if (-1 < i) {
          pops.push(d.userInfo);
        }
      }
    });

    if (0 < pushs.length) {
      return [...selectedUserList, ...pushs];
    }

    if (0 < pops.length) {
      return selectedUserList.filter(
        (u) => -1 === pops.findIndex((p) => p.seq === u.seq)
      );
    }
  }

  removedProfileSelection(
    userInfo: UserInfo,
    selectedUserList: UserInfoTypes[]
  ): UserInfoTypes[] {
    const i = selectedUserList.findIndex(
      (u) => String(u.seq) === String(userInfo.seq)
    );

    if (-1 < i) {
      return selectedUserList.filter(
        (u) => String(u.seq) !== String(userInfo.seq)
      );
    }
    return undefined;
  }

  extractBuddiesFromGroups(
    buddies: UserInfoF[],
    groups: GroupInfoDetail[]
  ): UserInfoF[] {
    if (
      !!buddies &&
      !!groups &&
      (buddies.length === 0 || groups.length === 0)
    ) {
      return;
    }
    const returnArr: UserInfoF[] = [];
    const groupUserSeqs: string[] = [];

    groups.map((g) => {
      g.userSeqs.map((seq) => {
        const find = groupUserSeqs.indexOf(seq);
        if (find < 0) {
          groupUserSeqs.push(seq);
        }
      });
    });

    buddies.map((buddy) => {
      const find = groupUserSeqs.indexOf(String(buddy.seq));
      if (find > -1) {
        returnArr.push(buddy);
      }
    });

    return returnArr;
  }

  private _updateMember(
    targetGroup: GroupInfoDetail,
    targetUserSeqs: string[]
  ) {
    this.store.dispatch(
      GroupActions.updateMember({
        targetGroup,
        targetUserSeqs
      })
    );
  }
}
