import { Subject } from 'rxjs';
import { take, takeUntil } from 'rxjs/operators';

import { Injectable, Inject, NgZone, OnDestroy } from '@angular/core';

import { MatDialog } from '@angular/material/dialog';

import { Store, select } from '@ngrx/store';

import { NativeService } from '@ucap/native';

import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';

import { AppAccountService } from './app-account.service';
import { AppAuthenticationService } from './app-authentication.service';

@Injectable({
  providedIn: 'root'
})
export class AppNativeService implements OnDestroy {
  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private appAccountService: AppAccountService,
    private appAuthenticationService: AppAuthenticationService,
    private store: Store<any>,
    private matDialog: MatDialog,
    private ngZone: NgZone
  ) {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  subscribe(): void {
    this.nativeService
      .app_onLogout$()
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(() => {
        this.ngZone.run(() => {
          this.matDialog.closeAll();

          this.appAuthenticationService.logoutConfirm().then((choice) => {
            if (choice) {
              this.appAuthenticationService.logout();
            }
          });
        });
      });

    this.nativeService
      .app_onStatus$()
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((statusCode) => {});

    this.nativeService
      .app_onShowSetting$()
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(() => {
        this.ngZone.run(() => {
          // this.store.dispatch(SettingsStore.showDialog());
          this.appAccountService.dialogForSettings();
        });
      });

    this.nativeService
      .app_onExit$()
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(() => {
        this.ngZone.run(() => {
          this.matDialog.closeAll();

          this.appAuthenticationService.logoutConfirm().then((choice) => {
            if (choice) {
              this.nativeService.app_exit();
            }
          });
        });
      });
  }
}
