import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import { Injectable, OnDestroy } from '@angular/core';

import { MatSidenav } from '@angular/material/sidenav';

import { Store, select } from '@ngrx/store';

import { AppSelector } from '@app/store/state';
import { WindowSizeMode } from '@app/types/window-size-mode.type';

@Injectable({
  providedIn: 'root'
})
export class AppUiService implements OnDestroy {
  set leftSidenav(leftSidenav: MatSidenav) {
    this._leftSidenav = leftSidenav;
    this._initLeftSideNav();
  }
  // tslint:disable-next-line: variable-name
  _leftSidenav: MatSidenav;

  // tslint:disable-next-line: variable-name
  private _windowSizeMode: WindowSizeMode;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(private store: Store<any>) {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AppSelector.windowSizeMode)
      )
      .subscribe((windowSizeMode) => {
        this._windowSizeMode = windowSizeMode;

        this._initLeftSideNav();
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  openLeftSidenavOnNarrowMode() {
    if (!this._leftSidenav) {
      return;
    }
    if (WindowSizeMode.narrow !== this._windowSizeMode) {
      return;
    }
    if (this._leftSidenav.opened) {
      return;
    }
    this._leftSidenav.open();
  }

  closeLeftSidenavOnNarrowMode() {
    if (!this._leftSidenav) {
      return;
    }
    if (WindowSizeMode.narrow !== this._windowSizeMode) {
      return;
    }
    if (!this._leftSidenav.opened) {
      return;
    }
    this._leftSidenav.close();
  }

  toggleLeftSidenavOnNarrowMode() {
    if (!this._leftSidenav) {
      return;
    }
    if (WindowSizeMode.narrow !== this._windowSizeMode) {
      return;
    }
    if (this._leftSidenav.opened) {
      this._leftSidenav.close();
    } else {
      this._leftSidenav.open();
    }
  }

  private _initLeftSideNav() {
    if (!!this._leftSidenav && !!this._windowSizeMode) {
      switch (this._windowSizeMode) {
        case WindowSizeMode.narrow:
          if (this._leftSidenav.opened) {
            this._leftSidenav.close();
          }
          this._leftSidenav.mode = 'over';

          break;
        default:
          if (!this._leftSidenav.opened) {
            this._leftSidenav.open();
          }
          this._leftSidenav.mode = 'side';
          break;
      }
    }
  }
}
