import { Subject } from 'rxjs';
import { tap } from 'rxjs/operators';

import { Injectable, Inject } from '@angular/core';

import { Store } from '@ngrx/store';
import { Actions, createEffect, ofType } from '@ngrx/effects';

import { NativeService } from '@ucap/native';

import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { LoginActions } from '@ucap/ng-store-authentication';

import { WindowSizeMode } from '@app/types/window-size-mode.type';

import { environment } from '@environments';

import {
  idleTimeChanged,
  windowResized,
  windowSizeModeChanged
} from './actions';

@Injectable()
export class Effects {
  webLoginSuccessForNativeService$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(LoginActions.webLoginSuccess),
        tap((action) => {
          this.nativeService.app_postLogin();
        })
      ),
    { dispatch: false }
  );

  loginSuccessForNativeService$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(LoginActions.loginSuccess),
        tap((action) => {
          if (!this.logoutSubject) {
            this.logoutSubject = new Subject();
          }
          // this.nativeService.idle_startCheck(1000);

          // this.nativeService
          //   .idle_onState$()
          //   .pipe(takeUntil(this.logoutSubject))
          //   .subscribe((idle) => {
          //     console.log('idle', idle);
          //   });
        })
      ),
    { dispatch: false }
  );

  logoutSuccessForNativeService$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(LoginActions.logoutSuccess),
        tap((action) => {
          this.logoutSubject.next();
          this.logoutSubject.complete();
          this.logoutSubject = undefined;

          // this.nativeService.idle_stopCheck();

          this.nativeService.app_postLogout();
        })
      ),
    { dispatch: false }
  );

  idleTimeChanged$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(idleTimeChanged),
        tap((action) => {
          this.nativeService.idle_changeLimitTime(action.idleTime);
        })
      ),
    { dispatch: false }
  );

  windowResized$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(windowResized),
        tap((action) => {
          const width = action.width;

          if (environment.productConfig.windowSizeMode.narrow > width) {
            // narrow mode
            this.store.dispatch(
              windowSizeModeChanged({
                windowSizeMode: WindowSizeMode.narrow
              })
            );
          } else if (
            environment.productConfig.windowSizeMode.narrow <= width &&
            environment.productConfig.windowSizeMode.normal > width
          ) {
            // normal mode
            this.store.dispatch(
              windowSizeModeChanged({
                windowSizeMode: WindowSizeMode.normal
              })
            );
          } else {
            // wide mode
            this.store.dispatch(
              windowSizeModeChanged({
                windowSizeMode: WindowSizeMode.wide
              })
            );
          }
        })
      ),
    { dispatch: false }
  );

  private logoutSubject: Subject<void>;

  constructor(
    private actions$: Actions,
    private store: Store<any>,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService
  ) {}
}
