import { map, tap, withLatestFrom } from 'rxjs/operators';

import { Injectable, Inject } from '@angular/core';
import { Router, NavigationEnd, PRIMARY_OUTLET } from '@angular/router';

import { Actions, createEffect, ofType } from '@ngrx/effects';

import { NativeService } from '@ucap/native';

import { LogService } from '@ucap/ng-logger';

import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';

import { RoomActions, ChattingSelector } from '@ucap/ng-store-chat';

import { Store, select } from '@ngrx/store';
import { SessionStorageService } from '@ucap/ng-web-storage';
import { AppKey } from '@app/types';
import { AppRoomSelector } from '../state';
import { AppChatService } from '@app/services/app-chat.service';

@Injectable()
export class Effects {
  createSuccess$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(RoomActions.createRoomSuccess),
        map((action) => action.res),
        tap((res) => {
          this.router.navigate(
            [
              'chat',
              {
                outlets: { content: 'chatroom' }
              }
            ],
            {
              queryParams: { roomId: res.roomId }
            }
          );
        })
      );
    },
    { dispatch: false }
  );

  selectedRoomSuccess$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(RoomActions.selectedRoomSuccess),
        tap((action) => {
          this.router.navigate(
            [
              'chat',
              {
                outlets: { content: 'chatroom' }
              }
            ],
            {
              queryParams: { roomId: action.roomId }
            }
          );
        })
      );
    },
    { dispatch: false }
  );

  clearSelectedRoom$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(RoomActions.clearSelectedRoom),
        withLatestFrom(this.store.pipe(select(AppRoomSelector.historyRoomId))),
        tap(([action, historyRoomId]) => {
          // HistoryRoomId clear.
          if (action.roomId === historyRoomId) {
            this.appChatService.setHistoryRoomId(null);

            let isChatMenu = this.router.url.startsWith('/chat/');

            if (!!isChatMenu) {
              // chat navigation init.
              this.router.navigate([
                'chat',
                {
                  outlets: { content: 'index' }
                }
              ]);
            }
          }
        })
      );
    },
    { dispatch: false }
  );

  constructor(
    private actions$: Actions,
    private store: Store<any>,
    private appChatService: AppChatService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private router: Router,
    private logService: LogService
  ) {}
}
