import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { CallHistory } from '@ucap/domain-call';

import { DateService } from '@ucap/ng-ui/date';
import { CallSelector } from '@ucap/ng-store-call';

import { SearchData } from '@app/ucap/organization/models/search-data';
import { Router } from '@angular/router';

@Component({
  selector: 'app-call-history-expansion',
  templateUrl: './history-expansion.component.html',
  styleUrls: ['./history-expansion.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class HistoryExpansionComponent implements OnInit, OnDestroy {
  @Input()
  set itemList(obj: CallHistory[]) {
    this._itemList = obj;
    this._initHistoryGroup();
  }
  get itemList(): CallHistory[] {
    return this._itemList;
  }
  _itemList: CallHistory[];

  historyGroup: { division: string; historyList: CallHistory[] }[];
  processing = false;

  private ngOnDestroySubject: Subject<void> = new Subject();
  constructor(
    private router: Router,
    private dateService: DateService,

    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  private _initHistoryGroup() {
    this.historyGroup = [];

    this.itemList.forEach((historyInfo) => {
      const date = historyInfo.startDate;
      let division = '';
      try {
        const value = this.dateService.get(date, 'YYYYMMDD');

        if (value === 'Invalid date') {
          division = date;
        } else {
          division = value;
        }
      } catch (error) {
        division = date;
      }

      const index = this.historyGroup.findIndex(
        (info) => info.division === division
      );
      if (index > -1) {
        this.historyGroup[index] = {
          ...this.historyGroup[index],
          historyList: [...this.historyGroup[index].historyList, historyInfo]
        };
      } else {
        this.historyGroup.push({
          division,
          historyList: [historyInfo]
        });
      }
    });

    this.changeDetectorRef.markForCheck();
  }

  onClickHistory(event: MouseEvent, historyInfo: CallHistory) {
    event.preventDefault();
    event.stopPropagation();

    let userSeq: string;
    if (!!historyInfo.sendYn && historyInfo.sendYn === true) {
      userSeq = historyInfo.calledUserSeq;
    } else {
      userSeq = historyInfo.callingUserSeq;
    }

    this.router.navigate(
      [
        'call',
        {
          outlets: { content: 'index' }
        }
      ],
      {
        queryParams: { id: userSeq }
      }
    );
  }
}
