import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input
} from '@angular/core';

import { ucapAnimations } from '@ucap/ng-ui/core';
import { CallHistory } from '@ucap/domain-call';
import { AppChatService } from '@app/services/app-chat.service';
import { MatDialog } from '@angular/material/dialog';
import {
  ProfileDialogComponent,
  ProfileDialogData,
  ProfileDialogResult
} from '@app/sections/organization/dialogs/profile.dialog.component';

@Component({
  selector: 'app-call-history-list-item-01',
  templateUrl: './history-list-item-01.component.html',
  styleUrls: ['./history-list-item-01.component.scss'],
  animations: ucapAnimations,
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class HistoryListItem01Component implements OnInit, OnDestroy {
  @Input()
  historyInfo: CallHistory;

  @Input()
  useFAB = false;

  isShowFAB = false;
  menuButtonType = 2;
  profileImageRoot: string;
  defaultProfileImage: string;

  private ngOnDestroySubject: Subject<void> = new Subject();
  constructor(
    private dialog: MatDialog,
    private appChatService: AppChatService,
    private changeDetectorRef: ChangeDetectorRef
  ) {
    this.defaultProfileImage = this.appChatService.defaultProfileImage;
    this.profileImageRoot = '';
  }

  ngOnInit(): void {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onMouseover(event: MouseEvent): void {
    if (!!this.useFAB) {
      this.isShowFAB = true;
    }
    event.preventDefault();
    event.stopPropagation();
  }
  onMouseleave(event: MouseEvent): void {
    if (!!this.useFAB) {
      this.isShowFAB = false;
    }

    event.preventDefault();
    event.stopPropagation();
  }

  getProfileImage(): string {
    if (!!this.historyInfo.sendYn && this.historyInfo.sendYn === true) {
      return this.historyInfo.calledImage;
    } else {
      return this.historyInfo.callingImage;
    }
  }

  openProfile(event: MouseEvent): void {
    event.preventDefault();
    event.stopPropagation();

    let userSeq: string;

    if (!!this.historyInfo.sendYn && this.historyInfo.sendYn === true) {
      userSeq = this.historyInfo.calledUserSeq;
    } else {
      userSeq = this.historyInfo.callingUserSeq;
    }

    if (!!userSeq) {
      this.dialog.open<
        ProfileDialogComponent,
        ProfileDialogData,
        ProfileDialogResult
      >(ProfileDialogComponent, {
        panelClass: 'mid-create-dialog',
        data: {
          userSeq
        }
      });
    }
  }
}
