import { Subject } from 'rxjs';
import moment from 'moment';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input
} from '@angular/core';
import { CallHistory } from '@ucap/domain-call';

@Component({
  selector: 'app-call-time-line',
  templateUrl: './time-line.component.html',
  styleUrls: ['./time-line.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class TimeLineComponent implements OnInit, OnDestroy {
  @Input()
  historyList: CallHistory[];

  private ngOnDestroySubject: Subject<void> = new Subject();
  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  /** Date Splitter show check */
  getDateSplitter(historyInfo: CallHistory): boolean {
    const curIndex = this.historyList.findIndex(
      (v) => v.callLogSeq === historyInfo.callLogSeq
    );

    if (curIndex === 0) {
      return true;
    } else if (curIndex > 0) {
      if (!this.historyList[curIndex]) {
        return false;
      }
      return !moment(this.historyList[curIndex].startDate).isSame(
        moment(this.historyList[curIndex - 1].startDate),
        'day'
      );
    }

    return false;
  }
}
