import { Subject, combineLatest } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { UserInfoSS } from '@ucap/domain-organization';
import { RoomInfo } from '@ucap/domain-chat';

import { RoomSelector } from '@ucap/ng-store-chat';

@Component({
  selector: 'app-chat-room-list',
  templateUrl: './room-list.component.html',
  styleUrls: ['./room-list.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class RoomListComponent implements OnInit, OnDestroy {
  @Input()
  set selectedRoom(roomInfo: RoomInfo) {
    this._selectedRoom = roomInfo;
    if (!!roomInfo) {
      this.selectedRoomList = [roomInfo];
    }
  }

  get selectedRoom() {
    return this._selectedRoom;
  }
  // tslint:disable-next-line: variable-name
  _selectedRoom: RoomInfo;

  @Input()
  checkable = false;

  @Input()
  isDialog = false;

  @Input()
  isMultiCheck = false;

  @Output()
  clicked: EventEmitter<{ userInfo: UserInfoSS }> = new EventEmitter();

  @Output()
  toggleRoom: EventEmitter<RoomInfo> = new EventEmitter();

  selectedRoomList: RoomInfo[] = [];

  roomList: RoomInfo[];

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {
    combineLatest([
      this.store.pipe(select(RoomSelector.rooms)),
      this.store.pipe(select(RoomSelector.standbyRooms))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([rooms, standbyRooms]) => {
        rooms = (rooms || []).filter((info) => {
          return (
            info.isJoinRoom &&
            !standbyRooms.find((standbyRoom) => standbyRoom === info.roomId)
          );
        });
        this.roomList = rooms;
        this.changeDetectorRef.markForCheck();
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onToggleRoom(event: { checked: boolean; roomInfo: RoomInfo }): void {
    if (!!this.isMultiCheck) {
    } else {
      if (!!event.checked) {
        this.selectedRoom = event.roomInfo;
        this.selectedRoomList = [this.selectedRoom];
      } else {
        this.selectedRoom = undefined;
        this.selectedRoomList = [];
      }
    }
    this.toggleRoom.emit(this.selectedRoom);
  }
}
