import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  EventEmitter,
  Output
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { UserInfoSS } from '@ucap/domain-organization';
import { StatusBulkInfo } from '@ucap/domain-status';

import { LogService } from '@ucap/ng-logger';
import { PresenceSelector } from '@ucap/ng-store-organization';
import { VersionInfo } from '@ucap/domain-authentication';

@Component({
  selector: 'app-group-profile-image-01',
  templateUrl: './profile-image-01.component.html',
  styleUrls: ['./profile-image-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileImage01Component implements OnInit, OnDestroy {
  @Input()
  set userInfo(userInfo: UserInfoSS) {
    this._userInfo = userInfo;
  }
  get userInfo(): UserInfoSS {
    return this._userInfo;
  }
  // tslint:disable-next-line: variable-name
  _userInfo: UserInfoSS;

  @Input()
  versionInfo: VersionInfo;

  set presenceInfo(p: StatusBulkInfo) {
    this._presenceInfo = p;
    this.changeDetectorRef.markForCheck();
  }
  get presenceInfo() {
    return this._presenceInfo;
  }
  // tslint:disable-next-line: variable-name
  _presenceInfo: StatusBulkInfo;

  @Output()
  openProfile: EventEmitter<UserInfoSS> = new EventEmitter();

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(
          PresenceSelector.selectStatusBulkInfo,
          Number(this.userInfo?.seq)
        )
      )
      .subscribe((status) => {
        this.presenceInfo = status;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onOpenProfile(userInfo: UserInfoSS) {
    this.openProfile.emit(userInfo);
  }
}
