import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { UserInfoF } from '@ucap/domain-organization';

import { LogService } from '@ucap/ng-logger';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import { VersionInfo } from '@ucap/domain-authentication';

interface CheckedInfo {
  checked: boolean;
  userInfo: UserInfoF;
}
@Component({
  selector: 'app-group-profile-list-01',
  templateUrl: './profile-list-01.component.html',
  styleUrls: ['./profile-list-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileList01Component implements OnInit, OnDestroy {
  @Input()
  checkable = false;

  @Input()
  isDialog = false;

  @Input()
  set userInfos(userInfos: UserInfoF[]) {
    this._userInfos = userInfos;
  }
  get userInfos(): UserInfoF[] {
    return this._userInfos;
  }
  // tslint:disable-next-line: variable-name
  _userInfos: UserInfoF[];

  @Input() selectedUser: UserInfoF[];

  @Output()
  clicked: EventEmitter<{ userInfo: UserInfoF }> = new EventEmitter();

  @Output()
  toggleCheck: EventEmitter<CheckedInfo[]> = new EventEmitter();

  versionInfo: VersionInfo;

  processing = false;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  /** 전체 체크여부 */
  checkAll() {
    if (!this.userInfos || 0 === this.userInfos.length) {
      return;
    }

    const checked: CheckedInfo[] = [];
    this.userInfos.forEach((u) => {
      checked.push({
        checked: true,
        userInfo: u
      });
    });
    this.toggleCheck.emit(checked);
  }

  uncheckAll() {
    if (!this.userInfos || 0 === this.userInfos.length) {
      return;
    }

    const unchecked: CheckedInfo[] = [];
    this.userInfos.forEach((u) => {
      unchecked.push({
        checked: false,
        userInfo: u
      });
    });
    this.toggleCheck.emit(unchecked);
  }

  /** 개별 체크여부 */
  getCheckedUser(userInfo: UserInfoF) {
    if (!!this.selectedUser && this.selectedUser.length > 0) {
      return (
        this.selectedUser.filter((item) => item.seq === userInfo.seq).length > 0
      );
    }
    return false;
  }

  /** 개별선택(토글) 이벤트 */
  onChangeCheckUser(param: { checked: boolean; userInfo: UserInfoF }) {
    this.toggleCheck.emit([param]);
    this.changeDetectorRef.detectChanges();
  }

  onClickUser(userInfo: UserInfoF): void {
    this.clicked.emit({ userInfo });
  }
}
