import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  EventEmitter,
  Output,
  ElementRef,
  Self
} from '@angular/core';

import { select, Store } from '@ngrx/store';

import { User, UserInfoF } from '@ucap/domain-organization';

import { I18nService } from '@ucap/ng-i18n';

import {
  ConfigurationSelector,
  AuthorizationSelector
} from '@ucap/ng-store-authentication';

import { ucapAnimations } from '@ucap/ng-ui/core';
import { UserSelector } from '@ucap/ng-store-organization';
import { VersionInfo } from '@ucap/domain-authentication';

@Component({
  selector: 'app-group-profile-list-item-04',
  templateUrl: './profile-list-item-04.component.html',
  styleUrls: ['./profile-list-item-04.component.scss'],
  animations: ucapAnimations,
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileListItem04Component implements OnInit, OnDestroy {
  @Input()
  userInfo: UserInfoF;

  @Input()
  checkable = false;

  @Input()
  checked = false;

  @Input()
  useFAB = false;
  @Input()
  menuButtonType = 0;

  @Output()
  changeCheck: EventEmitter<{
    checked: boolean;
    userInfo: UserInfoF;
  }> = new EventEmitter();

  isShowFAB = false;
  versionInfo: VersionInfo;
  // userPermission: UserPermission;
  user: User;
  statusMsgClass: string;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private changeDetectorRef: ChangeDetectorRef,
    private i18nService: I18nService,
    @Self() private elementRef: ElementRef,
    private store: Store<any>
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    // this.store
    //   .pipe(
    //     takeUntil(this.ngOnDestroySubject),
    //     select(AuthorizationSelector.userPermission)
    //   )
    //   .subscribe((userPermission) => {
    //     this.userPermission = userPermission;
    //   });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    this._userStatusMessage();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onToggleCheckUser(param: { checked: boolean; userInfo: UserInfoF }) {
    this.changeCheck.emit(param);
    this.changeDetectorRef.detectChanges();
  }

  onMouseover(event: MouseEvent): void {
    if (!!this.useFAB) {
      this.isShowFAB = true;
    }
    event.preventDefault();
    event.stopPropagation();
  }
  onMouseleave(event: MouseEvent): void {
    if (!!this.useFAB) {
      this.isShowFAB = false;
    }

    event.preventDefault();
    event.stopPropagation();
  }

  _userStatusMessage(): string {
    if (
      !!this.userInfo &&
      !!this.userInfo.nickName &&
      this.userInfo.nickName.trim() !== ''
    ) {
      this.statusMsgClass = 'intro-name';
      return this.userInfo.nickName;
    } else if (
      !!this.userInfo &&
      !!this.userInfo.intro &&
      this.userInfo.intro.trim() !== ''
    ) {
      this.statusMsgClass = 'intro';
      return this.userInfo.intro;
    } else {
      this.statusMsgClass = null;
      return '';
    }
  }
}
