// This file can be replaced during build by using the `fileReplacements` array.
// `ng build --prod` replaces `environment.ts` with `environment.prod.ts`.
// The list of file replacements can be found in `angular.json`.
import {
  DeviceType,
  DesktopType,
  NotificationMethod
} from '@ucap/domain-common';
import { NativeService, I18NEXT } from './native/native';

import {
  Environment,
  commonApiUrls,
  commonApiacceptableFileExtensions,
  publicApiUrls,
  externalApiUrls,
  contactApiUrls,
  piUrls,
  protocolUrls,
  messageApiUrls,
  promptUrls,
  commonApiAcceptableFileExtensionsForImage,
  commonApiAcceptableFileExtensionsForVideo,
  webexUrls
} from './environment.type';

export const environment: Environment = {
  production: false,
  hmr: false,

  productConfig: {
    productId: 'PRO_000482',
    productName: 'MMessenger',
    localEncriptionKey: '!@#$LG%^&*',
    windowSizeMode: {
      narrow: 780,
      normal: 1024
    },
    nativeServiceClass: NativeService,
    i18next: {
      options: {
        whitelist: ['ko', 'en'],
        fallbackLng: 'en',
        debug: true,
        returnEmptyString: false,
        ns: [
          'common',
          'organization',
          'authentication',
          'group',
          'locale',
          'chat',
          'call',
          'message'
        ],
        backend: {
          loadPath: `${I18NEXT.loadPathRoot}assets/i18n/{{lng}}/{{ns}}.json`
        }
      },
      useBackends: I18NEXT.backends
    },
    supportedLanguages: ['ko', 'en'],
    updateCheck: {
      deviceType: DeviceType.Renderer,
      devicePlatformType: DesktopType.Windows,
      intervalHour: 1
    },
    defaultSettings: {
      general: {
        theme: 'theme-default',
        autoLogin: false,
        autoLaunch: false,
        continueRunWhenClose: true,
        locale: 'ko',
        hrInfoLocale: 'ko',
        startupHideWindow: false,
        timezone: 'Asia/Seoul'
      },
      notification: {
        preView: true,
        use: true,
        method: NotificationMethod.SoundAndAlert,
        alertExposureTime: 5,
        receiveForMobile: false,
        receiveForMessage: false
      },
      chat: {
        fontFamily: 'Malgun Gothic',
        fontSize: 12,
        downloadPath: undefined
      },
      presence: {
        absenceTime: 10
      }
    },
    file: {
      defaultDownloadFolder: 'MMessenger Download',
      defaultFileAllowSize: 100
    },
    authentication: {
      usePrivateInformationAgree: false,
      useRememberMe: true,
      useAutoLogin: true,
      maxTryCount: 5,
      intervalForRetry: 1 * 60
    },
    profile: {
      editableProfileImage: false
    },
    organization: {
      displayRoot: false,
      supportedLanguages: ['ko', 'en']
    },
    group: {
      useMyDeptGroup: false,
      myDeptGroupSeq: -5,
      fixedGroupSeqs: [-5]
    },
    chat: {
      useTimerRoom: false,
      timerRoomDefaultInterval: 24 * 60 * 60,

      maxChatRoomUser: 300,
      masstextLength: 800,

      eventRequestInitCount: 30,
      eventRequestDefaultCount: 50,

      readHereShowMinimumEventCount: 10,
      readHereShowMaximumEventCount: 100,
      readHereEventRequestCount: 10
    },
    call: {
      historyRequestDefaultCount: 200
    }
  },

  companyConfig: {
    companyGroupCode: 'LG',
    fixedCompanyCode: null
  },

  commonApiModuleConfig: {
    hostConfig: {
      protocol: 'http',
      domain: '13.124.88.127',
      port: 8011
    },
    urls: commonApiUrls,
    acceptableFileExtensions: commonApiacceptableFileExtensions,
    acceptableFileExtensionsForImage: commonApiAcceptableFileExtensionsForImage,
    acceptableFileExtensionsForVideo: commonApiAcceptableFileExtensionsForVideo
  },

  publicApiModuleConfig: {
    hostConfig: {
      protocol: 'http',
      domain: '13.124.88.127',
      port: 8011
    },
    urls: publicApiUrls
  },

  externalApiModuleConfig: {
    hostConfig: {
      protocol: 'http',
      domain: '13.124.88.127',
      port: 8011
    },
    urls: externalApiUrls
  },

  contactApiModuleConfig: {
    hostConfig: {
      protocol: 'http',
      domain: '13.124.88.127',
      port: 8011
    },
    urls: contactApiUrls
  },

  messageApiModuleConfig: {
    hostConfig: {
      protocol: 'http',
      domain: '13.124.88.127',
      port: 9097
    },
    urls: messageApiUrls
  },

  promptApiModuleConfig: {
    hostConfig: {
      protocol: 'http',
      domain: '13.124.88.127',
      port: 9097
    },
    urls: promptUrls
  },

  webexApiModuleConfig: {
    hostConfig: {
      protocol: 'http',
      domain: '13.124.88.127',
      port: 9098
    },
    urls: webexUrls
  },

  piModuleConfig: {
    hostConfig: {
      protocol: 'http',
      domain: '13.124.88.127',
      port: 9097
    },
    urls: piUrls
  },

  webSocketModuleConfig: {
    reconnect: {
      delay: 1000 * 3
    }
  },

  protocolModuleConfig: {
    hostConfig: {
      protocol: 'ws',
      domain: '13.124.88.127',
      port: 8080
    },
    urls: protocolUrls,
    requestId: {
      min: 1,
      max: 59999
    },
    timeout: {
      expiredTime: 15,
      checkInterval: 1
    }
  },
  pingProtocolModuleConfig: {
    statusCode: 'O',
    interval: 5 * 60
  }
};

/*
 * For easier debugging in development mode, you can import the following file
 * to ignore zone related error stack frames such as `zone.run`, `zoneDelegate.invokeTask`.
 *
 * This import should be commented out in production mode because it will have a negative impact
 * on performance if an error is thrown.
 */
// import 'zone.js/dist/zone-error';  // Included with Angular CLI.
